#!/bin/sh
#run this script after running 1kernel.sh, then after a reboot.
#110827 compile commercial nvidia modules.
#110914 tweaks.
#120125 check nvidia dir exists.
#150712 no longer using ndiswrapper. 150917 bring it back, improve install. remove again.
#160323 qca9377 driver for wifi, my new netbook. 160324 removed.
#170707 fix paths for ndiswrapper. note: it doesn't compile with 4.11.9.
#170719 add driver for rtl8723bu wifi, for alpha litebook laptop. 170720
#20201202 wl.ko driver for broadcom wifi. note, this is alternative to other drivers in kernel.
#20220306 put $TODAYDATE in broadcom pet name.
#20220403 rtw89 realtek wifi driver. note: mainlined in 5.16 kernel.
#20220403 realtek rtl8411 card reader driver. ref: https://github.com/M0Rf30/rts_bpp
#20220403 delay getting /lib/firmware from 1kernel.sh until 2modules.sh
#20220426 new droidcam v4l2loopback-dc.ko
#20220516 make sure TODAYDATE same all scripts.
#20240205 updated broadcom wl driver for 6.6 kernel.

. /etc/DISTRO_SPECS

KERNELVERSION="`uname -r`"
[ ! -d linux-${KERNELVERSION} ] && exit
#TODAYDATE="`date +%Y%m%d`"
TODAYDATE="$(cat TODAYDATE)" #20220516

echo "System currently running with ${KERNELVERSION} kernel."
echo "The modules will be compiled for this kernel."
echo

DASHOUT="`find /usr/src -maxdepth 1 -type d -name "out-k${KERNELVERSION}-*"`"
if [ "$DASHOUT" ];then
 POSTVER="-`echo -n "$DASHOUT" | rev | cut -f 1 -d '-' | rev`" #ex: -43
else
 POSTVER=""
fi

MACHINENAME="`uname -m`" #ex: x86_64

#3rd-party drivers

#ndiswrapper  150712 170707
NDISWRAPPERTB="`find 3rd-party/wireless -maxdepth 1 -type f -name 'ndiswrapper*.tar.gz'`"
if [ "$NDISWRAPPERTB" != "" ];then
 cd 3rd-party/wireless
 # svn co https://ndiswrapper.svn.sourceforge.net/svnroot/ndiswrapper/trunk ndiswrapper
 # mv -f ndiswrapper ndiswrapper-${TODAYDATE}
 # NDISWRAPPERDIR=ndiswrapper-${TODAYDATE}
 #else
 NDISWRAPPERDIR="`basename $NDISWRAPPERTB .tar.gz`"
 tar -xf ${NDISWRAPPERDIR}.tar.gz
 cd $NDISWRAPPERDIR
 #cd ndiswrapper
 make
 make install
 #... misc/ndiswrapper.ko
 #... note, there is also userland stuff.
 cd ../
 rm -rf ./$NDISWRAPPERDIR
 #mv -f ./${NDISWRAPPERDIR}-i486 /usr/src/out-k${KERNELVERSION}${POSTVER}/
 mkdir -p /usr/src/out-k${KERNELVERSION}${POSTVER}/${NDISWRAPPERDIR}-${MACHINENAME}/sbin
 mkdir -p /usr/src/out-k${KERNELVERSION}${POSTVER}/${NDISWRAPPERDIR}-${MACHINENAME}/usr/sbin
 [ -e /sbin/loadndisdriver ] && cp -a /sbin/loadndisdriver /usr/src/out-k${KERNELVERSION}${POSTVER}/${NDISWRAPPERDIR}-${MACHINENAME}/sbin/
 [ -e /usr/sbin/ndiswrapper ] && cp -a /sbin/loadndisdriver /usr/src/out-k${KERNELVERSION}${POSTVER}/${NDISWRAPPERDIR}-${MACHINENAME}/usr/sbin/
 
 cd ../../
fi

echo -n "Press ENTER: " ; read GOON

#sysprof...
cd 3rd-party/misc
SRCPKG="`find . -maxdepth 1 -type f -name 'sysprof*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  ./configure --prefix=/usr --sysconfdir=/etc --localstatedir=/var
  make
  make install
  #...installs extra/sysprof-module.ko plus some userland stuff.
  sync
  cd ..
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../


#orinoco...
cd 3rd-party/wireless
SRCPKG="`find . -maxdepth 1 -type f -name 'orinoco_usb*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 if [ -d orinoco_usb ];then
  cd orinoco_usb
  make
  make install
  #...installs to kernel/drivers/net/wireless/orinoco/
  sync
  cd ..
  rm -rf orinoco_usb
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../


#ltmodem...
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'ltmodem*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  make
  #manual install of ltmodem.ko, ungrab-serial.ko and v8250.ko to /lib/modules/2.6.32.38/ltmodem
  mkdir /lib/modules/${KERNELVERSION}/ltmodem
  cp -f ltmodem.ko /lib/modules/${KERNELVERSION}/ltmodem/
  cp -f ungrab-serial.ko /lib/modules/${KERNELVERSION}/ltmodem/
  cp -f v8250.ko /lib/modules/${KERNELVERSION}/ltmodem/
  sync
  cd ..
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../


#pctel...
#note, source patched by me, fix pci ids (file ptserial_pci-2.6.c).
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'pctel*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  cd src
  echo "pct789" | ./configure -manual
  make
  make install
  #... misc/linmodem.ko, pctel_hw.ko, pctel.ko (note ess will overwrite linmodem.ko)
  sync
  cd ../../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../


#ess
#Got source from here: http://tx.technion.ac.il/~raindel/
#note, configure script patched so will still compile if modem card not present.
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'ess_2.6*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  ./configure
  make
  #created linmodems.ko and esscom.ko, but reluctant to run 'make install'
  #as overwrites linmodems.ko installed by pctel driver -- although Jeff trull
  #tells me they are the same. -- in that case, I'll keep the pctel version.
  #Manual copy esscom.ko and esscom_hw.ko to /lib/modules/2.6.31.14/misc
  mkdir -p /lib/modules/${KERNELVERSION}/misc
  cp -f esscom_hw.ko /lib/modules/${KERNELVERSION}/misc/
  cp -f esscom.ko /lib/modules/${KERNELVERSION}/misc/
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#slmodem
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'slmodem*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  cd drivers
  make clean
  make
  mkdir -p /lib/modules/${KERNELVERSION}/slmodem
  cp -f slusb.ko /lib/modules/${KERNELVERSION}/slmodem/
  cp -f slamr.ko /lib/modules/${KERNELVERSION}/slmodem/
  sync
  cd ../../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#ungrab-winmodem...
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'ungrab-winmodem*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  make
  make install
  #...extra/ungrab-winmodem.ko
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#intel
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'intel*.tar.bz2'`"
if [ "$SRCPKG" ];then
 mkdir /lib/modules/${KERNELVERSION}/intelmodem
 mkdir /lib/modules/${KERNELVERSION}/intelmodem/536EP
 mkdir /lib/modules/${KERNELVERSION}/intelmodem/537
 mkdir /lib/modules/${KERNELVERSION}/intelmodem/537EP
 mkdir /lib/modules/${KERNELVERSION}/intelmodem/537SP
 mkdir /lib/modules/${KERNELVERSION}/intelmodem/537EA
 tar -jxf $SRCPKG
 if [ -d intel-536-537 ];then
  cd intel-536-537
  cd coredrv
  make INTEL_MODEM=536EP 536core_26
  cp -f Intel536.ko to /lib/modules/${KERNELVERSION}/intelmodem/536EP/
  make clean
  make INTEL_MODEM=537 537=537 537core_26
  cp -f Intel537.ko /lib/modules/${KERNELVERSION}/intelmodem/537/Intel537.HIDE
  make clean
  make INTEL_MODEM=537EP 537=537 537core_26
  cp -f Intel537.ko /lib/modules/${KERNELVERSION}/intelmodem/537EP/Intel537.HIDE
  make clean
  make INTEL_MODEM=537SP 537=537 537core_26
  cp -f Intel537.ko /lib/modules/${KERNELVERSION}/intelmodem/537SP/Intel537.HIDE
  make clean
  make INTEL_MODEM=537EA 537=537 537core_26
  cp -f Intel537.ko /lib/modules/${KERNELVERSION}/intelmodem/537EA/Intel537.HIDE
  sync
  cd ../../
  rm -rf intel-536-537
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#agrsm 11c11040
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'agrsm-11c*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  make
  #...manual install agrmodem.ko, agrserial.ko to /lib/modules/2.6.31.14/agrmodem/11c11040/
  mkdir /lib/modules/${KERNELVERSION}/agrmodem
  mkdir /lib/modules/${KERNELVERSION}/agrmodem/11c11040/
  cp -f agrmodem.ko /lib/modules/${KERNELVERSION}/agrmodem/11c11040/
  cp -f agrserial.ko /lib/modules/${KERNELVERSION}/agrmodem/11c11040/
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#agrsm 06pci
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'agrsm06pci*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  make
  mkdir -p /lib/modules/${KERNELVERSION}/agrmodem
  mkdir /lib/modules/${KERNELVERSION}/agrmodem/06pci/
  cp -f agrmodem.ko /lib/modules/${KERNELVERSION}/agrmodem/06pci/agrmodem.HIDE
  cp -f agrserial.ko /lib/modules/${KERNELVERSION}/agrmodem/06pci/agrserial.HIDE
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#agrsm 048pci
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'agrsm048pci*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  make
  mkdir -p /lib/modules/${KERNELVERSION}/agrmodem
  mkdir /lib/modules/${KERNELVERSION}/agrmodem/048pci/
  cp -f agrmodem.ko /lib/modules/${KERNELVERSION}/agrmodem/048pci/agrmodem.HIDE
  cp -f agrserial.ko /lib/modules/${KERNELVERSION}/agrmodem/048pci/agrserial.HIDE
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#dgcmodem
#Conexant driver for USB hardware modems. This has been worked out by rerwin...
#Note, The Conexant drivers need the full depmod.
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'dgcmodem*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  depmod #depmod is the full version.
  make install
  echo "" | dgcconfig --kernel
  #...installed /lib/modules/${KERNELVERSION}/extra/dgcusbdcp.ko
  #plus userland stuff.
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#hsfmodem
#Conexant hsf modem drivers. Special source package created by rerwin.
cd 3rd-party/modems
SRCPKG="`find . -maxdepth 1 -type f -name 'hsfmodem*.tar.gz'`"
if [ "$SRCPKG" ];then
 tar -zxf $SRCPKG
 BASENAME="`basename $SRCPKG .tar.gz`"
 if [ -d $BASENAME ];then
  cd $BASENAME
  depmod #depmod is the full version.
  make install
  echo -e 'no\n\n' | hsfconfig --kernel
  #...installed /lib/modules/${KERNELVERSION}/extra/, but manually move hsf*.ko into hsfmodem directory.
  #plus userland stuff.
  mkdir -p /lib/modules/${KERNELVERSION}/hsfmodem
  mv -f /lib/modules/${KERNELVERSION}/extra/hsf*.ko /lib/modules/${KERNELVERSION}/hsfmodem/
  sync
  cd ../
  rm -rf $BASENAME
 fi
 echo -n "Press ENTER: " ; read GOON
fi
cd ../../

#hcfpcimodem 1.21full
#--------------------
#Conexant hsf modem drivers. Instructions from rerwin.
#leaving this out, as it is speed-limited to 14.4KB.

#martian
#don't know.

#110827 nvidia
if [ -d 3rd-party/nvidia ];then #120125
 cd 3rd-party/nvidia
 export SYSSRC=/usr/src/linux-${KERNELVERSION}
 export KERNEL_MODLIB=/lib/modules/${KERNELVERSION}
 export KERNEL_UNAME=${KERNELVERSION}
 export MODULE_ROOT=/lib/modules/${KERNELVERSION}
 export SYSOUT=/usr/src/linux-${KERNELVERSION}
 export CC=cc
 for ONENV in `ls -1 | tr '\n' ' '` #ex: NVIDIA-Linux-x86-195.36.31-pkg1.run NVIDIA-Linux-x86-260.19.12.run NVIDIA-Linux-x86-71.86.14-pkg1.run
 do
  sh ./${ONENV} --extract-only
  if [ $? -eq 0 ];then
   NVBASE="`basename $ONENV .run`"
   NVVER="`echo -n "$ONENV" | cut -f 4 -d '-'`" #ex: 195.36.31
   NVVER="`basename $NVVER .run`" #one case, still has .run on end.
   if [ -d ${NVBASE}/usr/src/nv ];then
    cd ${NVBASE}/usr/src/nv
    make module
    sync
    mkdir -p ../../../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia
    cp nvidia.ko ../../../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia/
    SIZEK="`du -k -s ../../../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION} | cut -f 1`"
    #db fields: pkgname|nameonly|version|pkgrelease|category|size|path|fullfilename|dependencies|description|compileddistro|compiledrelease|repo|
    echo "nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}|nvidia_kmodule|${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}||Setup|${SIZEK}K||nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}.pet|+linux_kernel&eq${KERNELVERSION},+nvidia_base&eq${NVVER}|commercial kernel nvidia driver, needs nvidia_base pkg, runs on kernel ${KERNELVERSION}, compiled in ${DISTRO_NAME} version ${DISTRO_VERSION}|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > ../../../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/pet.specs
    cd ../../../../
   else
    cd ${NVBASE}/kernel
    make module
    sync
    mkdir -p ../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia
    cp nvidia.ko ../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION}/nvidia/
    SIZEK="`du -k -s ../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/lib/modules/${KERNELVERSION} | cut -f 1`"
    #db fields: pkgname|nameonly|version|pkgrelease|category|size|path|fullfilename|dependencies|description|compileddistro|compiledrelease|repo|
    echo "nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}|nvidia_kmodule|${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}||Setup|${SIZEK}K||nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}.pet|+linux_kernel&eq${KERNELVERSION},+nvidia_base&eq${NVVER}|commercial kernel nvidia driver, needs nvidia_base pkg, runs on kernel ${KERNELVERSION}, compiled in ${DISTRO_NAME} version ${DISTRO_VERSION}|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > ../../../../out-k${KERNELVERSION}${POSTVER}/nvidia_kmodule-${NVVER}-k${KERNELVERSION}-${DISTRO_COMPAT_VERSION}/pet.specs
    cd ../../
   fi
   sync
   rm -rf ./${NVBASE}
  fi
 done
 cd ../../
fi

#160323 160324
#if [ -f 3rd-party/wireless/Qualcomm-Atheros-QCA9377-Wifi-Linux-master.tar.gz ];then
# echo "Compiling Qualcomm-Atheros-QCA9377-Wifi driver..."
# cp -a -f 3rd-party/wireless/Qualcomm-Atheros-QCA9377-Wifi-Linux-master.tar.gz ./
# tar -xf Qualcomm-Atheros-QCA9377-Wifi-Linux-master.tar.gz
# cd Qualcomm-Atheros-QCA9377-Wifi-Linux-master
# ./install.sh
# sync
# cd ..
# rm -rf Qualcomm-Atheros-QCA9377-Wifi-Linux-master
# rm -f Qualcomm-Atheros-QCA9377-Wifi-Linux-master.tar.gz
#fi

#170719 realtek 8723bu wifi driver, for alpha litebook
RTL8723BU="$(find 3rd-party/wireless -maxdepth 1 -type f -name 'rtl8723bu*tar.gz')"
if [ "$RTL8723BU" ];then
 echo "Compiling rtl8723bu wifi driver..."
 NAMEONLY="$(basename $RTL8723BU .tar.gz)"
 cd 3rd-party/wireless
 tar -xf ${NAMEONLY}.tar.gz
 cd ${NAMEONLY}
 #170720 disable concurrent-mode, it causes wlan0 and wlan1 which is confusing...
 sed -i '/DCONFIG_CONCURRENT_MODE/d' Makefile
 make
 make install
 sync
 cd ..
 rm -rf ./${NAMEONLY}
 cd ../../
fi

#20220403 
RTW89="$(find 3rd-party/wireless -maxdepth 1 -type f -name 'rtw89*tar.gz')"
if [ "$RTW89" ];then
 echo "Compiling rtw89 wifi driver..."
 NAMEONLY="$(basename $RTW89 .tar.gz)"
 cd 3rd-party/wireless
 tar -xf ${NAMEONLY}.tar.gz
 cd ${NAMEONLY}
 make
 make install
 sync
 cd ..
 rm -rf ./${NAMEONLY}
 cd ../../
fi

#20220403 
RTL8411="$(find 3rd-party/misc -maxdepth 1 -type f -name 'rts_bpp*tar.gz')"
if [ "$RTL8411" ];then
 echo "Compiling rts_bpp rtl8411 card reader driver..."
 NAMEONLY="$(basename $RTL8411 .tar.gz)"
 cd 3rd-party/misc
 tar -xf ${NAMEONLY}.tar.gz
 cd ${NAMEONLY}
 make
 #Makefile cannot install...
 if [ -f rts_bpp.ko ];then
  #dunno why putting it here, google search found others put it here...
  mkdir -p /lib/modules/${KERNELVERSION}/kernel/drivers/scsi
  cp -a rts_bpp.ko /lib/modules/${KERNELVERSION}/kernel/drivers/scsi/
 else
  echo "ERROR rts_bpp compile fail."
 fi
 sync
 cd ..
 rm -rf ./${NAMEONLY}
 cd ../../
fi

#20220426 
DROIDCAM="$(find 3rd-party/misc -maxdepth 1 -type f -name 'droidcam*tar.gz')"
if [ "$DROIDCAM" ];then
 echo 'Compiling droidcam v4l2loopback-dc driver...'
 NAMEONLY="$(basename $DROIDCAM .tar.gz)"
 cd 3rd-party/misc
 tar -xf ${NAMEONLY}.tar.gz
 cd ${NAMEONLY}
 make
 if [ -f v4l2loopback-dc.ko ];then
  make install
  #...no, that installs to /lib/modules/${KERNELVERSION}/extra
  #mkdir -p /lib/modules/${KERNELVERSION}/kernel/drivers/media/video
  #cp -a -f v4l2loopback-dc.ko /lib/modules/${KERNELVERSION}/kernel/drivers/media/video/
  #...have left it at 'make install' as think that signs the module.
  # yes, ref: https://www.kernel.org/doc/html/v4.15/admin-guide/module-signing.html
 fi
 sync
 cd ..
 rm -rf ./${NAMEONLY}
 cd ../../
fi

#20201202  20240205 ref: https://github.com/antoineco/broadcom-wl
STA_WL="$(find 3rd-party/wireless -maxdepth 1 -type f -name 'broadcom-wl-*z')"
if [ "$STA_WL" ];then
 echo "Compiling Broadcom STA WL driver (wl.ko)..."
 PKGNAME="$(basename $STA_WL)"
 cd 3rd-party/wireless
 tar -xf $PKGNAME
 if [ -d broadcom-wl-6.6 ];then
  cd broadcom-wl-6.6
  make
  sync
  if [ -f wl.ko ];then
   #create a separate PET for this...
   mkdir -p broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/lib/modules/${KERNELVERSION}/kernel/drivers/net/wireless
   cp -a -f wl.ko broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/lib/modules/${KERNELVERSION}/kernel/drivers/net/wireless/
   echo '#!/bin/sh
if [ "`pwd`" == "/" ];then
 depmod
fi' > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pinstall.sh
   chmod 755 broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pinstall.sh
   echo '#!/bin/sh
depmod' > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/puninstall.sh
   chmod 755 broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/puninstall.sh
   echo "broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64|broadcom-sta-wl|20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64||BuildingBlock|7380K||broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet|+linux_kernel&eq${KERNELVERSION}|Broadcom wl.ko alternative driver|${DISTRO_BINARY_COMPAT}|${DISTRO_COMPAT_VERSION}||" > broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64/pet.specs
   dir2tgz broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64
   tgz2pet broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.tar.gz
   mv -f broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet ../../../out-k${KERNELVERSION}/
   mv -f broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64 ../../../out-k${KERNELVERSION}/
   echo "PET created: out-k${KERNELVERSION}/broadcom-sta-wl-20201202-k${KERNELVERSION}-${TODAYDATE}-${DISTRO_COMPAT_VERSION}64.pet"
   echo "...wl.ko will not be included in main PET, as conflicts with other drivers."
  else
   echo "...Sorry, compile failed, wl.ko not created"
  fi
  sync
  cd ..
  rm -rf broadcom-wl-6.6
 else
  echo "...Sorry, directory 'broadcom-wl-6.6' does not exist, cannot compile."
 fi
 cd ../../
fi

#20220403 rtw89 has installed some firmware, code delayed from 1kernel.sh...
echo "Creating 'firmware' tarball in /usr/src/out-k${KERNELVERSION}${POSTVER}..."
cp -a -f /lib/firmware out-k${KERNELVERSION}${POSTVER}/
cd out-k${KERNELVERSION}${POSTVER}
dir2tgz firmware/
sync
rm -rf firmware
cd ..

depmod
sync
echo "Current directory: `pwd`"
echo -n "Press ENTER to continue: "
read keepgoing

echo "Creating tarball of modules, out-k${KERNELVERSION}${POSTVER}/${KERNELVERSION}.tar.gz ..."
cp -a -f /lib/modules/${KERNELVERSION} out-k${KERNELVERSION}${POSTVER}/
cd out-k${KERNELVERSION}${POSTVER}
[ -f ${KERNELVERSION}.tar.gz ] && mv -f ${KERNELVERSION}.tar.gz ${KERNELVERSION}.tar.gzWITHOUT_3RDPARTY
dir2tgz ${KERNELVERSION}
sync
rm -rf ${KERNELVERSION}
cd ..

#110827
echo
echo "make commercial nvidia modules into pets..."
cd out-k${KERNELVERSION}${POSTVER}
for ONENV in `find . -maxdepth 1 -type d -name 'nvidia_kmodule*' | sed -e 's%^\./%%' | tr '\n' ' '`
do
 dir2tgz $ONENV
 tgz2pet ${ONENV}.tar.gz
 sync
done
cd ..

echo
echo "Finished. Now run 3package.sh"

###END###
