#!/bin/sh

NVRUN="$1"
[ -z "$NVRUN" ] && exit 1
[ ! -f "$NVRUN" ] && exit 1
KERNELVER="$(uname -r)"
NVVER="$(echo -n "$NVRUN" | cut -f 4 -d '-' | sed 's%\.run%%')"

chmod 755 ${NVRUN} #NVIDIA-Linux-x86_64-${NVVER}.run

. /etc/DISTRO_SPECS
. /etc/rc.d/PUPSTATE

case "$NVRUN" in
 NVIDIA-Linux-x86_64-390*)
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --no-unified-memory --concurrency-level=1 --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
 NVIDIA-Linux-x86_64-470*) #20240826
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --concurrency-level=1 --skip-depmod --no-systemd --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
 *)
  #20231116 remove:  --no-peermem --no-unified-memory
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --no-dkms --concurrency-level=1 --skip-depmod --no-systemd --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
esac

if [ -d out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64 ];then
 rm -rf out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64
fi
rm -f out-k${KERNELVER}/nvidia_* 2>/dev/null
sync

mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/bin
mkdir out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib
mkdir out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share
cp -a /mnt/.easy_rw/mainrw/usr/bin/nvidia-* out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/bin/
cp -a /mnt/.easy_rw/mainrw/usr/lib/*.so* out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/
cp -a /mnt/.easy_rw/mainrw/usr/lib/gbm out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/

mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER}/kernel/drivers
cp -a /mnt/.easy_rw/mainrw/usr/lib/modules/${KERNELVER}/kernel/drivers/video out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER}/kernel/drivers/

cp -a /mnt/.easy_rw/mainrw/usr/lib/nvidia out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/
cp -a /mnt/.easy_rw/mainrw/usr/lib/vdpau out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/
cp -a /mnt/.easy_rw/mainrw/usr/lib/xorg out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/

cp -a /mnt/.easy_rw/mainrw/usr/share/egl out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/
cp -a /mnt/.easy_rw/mainrw/usr/share/glvnd out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/
cp -a /mnt/.easy_rw/mainrw/usr/share/nvidia out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/
cp -a /mnt/.easy_rw/mainrw/usr/share/X11 out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/

mkdir out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/firmware
cp -a /mnt/.easy_rw/mainrw/usr/lib/firmware/nvidia out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/firmware/


mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/modprobe.d
echo 'blacklist i915
install i915 /bin/false
blacklist nouveau
install nouveau /bin/false' > out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/modprobe.d/nvidia.conf

SFSPATH="/mnt/build/bk/www/ibiblio/easyos/${DISTRO_TARGETARCH}/packages/sfs/easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}"
if [ ! -d ${SFSPATH} ];then
 echo "Path ${SFSPATH} does not exist"
 exit 4
fi

#create sfs...
cd out-k${KERNELVER}
echo "Creating meta-data for nvidia_${KERNELVER}-${NVVER}_amd64.sfs ..."
SIZEK="$(du -s -k nvidia_${KERNELVER}-${NVVER}_amd64 | cut -f 1)"
echo "#generated by 5nvidia.sh:
DB_pkgname='nvidia_${KERNELVER}-${NVVER}_amd64'
DB_nameonly='nvidia'
DB_version='${KERNELVER}'
DB_revision='${NVVER}'
DB_arch='amd64'
DB_categories='Utility'
DB_size='${SIZEK}'
DB_fullfilename='nvidia_${KERNELVER}-${NVVER}_amd64.sfs'
DB_dependencies='easy'
DB_description='Official NVIDIA GPU support'
DB_description_long='Kernel modules and libraries for NVIDIA GPUs. 390 series is for older GPUs, 525 and later for newer.'
DB_compileddistro='${DISTRO_BINARY_COMPAT}'
DB_compiledrelease='${DISTRO_COMPAT_VERSION}'
DB_path='easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}'
DB_apps='nvidia'
DB_icons='nvidia'
DB_install='mainonly'
DB_hash=''" > nvidia_${KERNELVER}-${NVVER}_amd64.specs

mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/.control
cp -a nvidia_${KERNELVER}-${NVVER}_amd64.specs nvidia_${KERNELVER}-${NVVER}_amd64/.control/nvidia.specs
mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps

cp -a ../nvidia.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/
cp -a ../nvidia.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png

sync
mksquashfs nvidia_${KERNELVER}-${NVVER}_amd64 nvidia_${KERNELVER}-${NVVER}_amd64.sfs -comp zstd
sync
chmod 644 nvidia_${KERNELVER}-${NVVER}_amd64.sfs

HASH="$(sha256sum nvidia_${KERNELVER}-${NVVER}_amd64.sfs | cut -f 1 -d ' ')"
sed -i -e "s%^DB_hash=.*%DB_hash='${HASH}'%" nvidia_${KERNELVER}-${NVVER}_amd64.specs

echo -n "Press ENTER to install SFS: "
read goon
echo
echo "nvidia_${KERNELVER}-${NVVER}_amd64.sfs created.
Installed at: ${SFSPATH}"
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.sfs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.specs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia.png ${SFSPATH}/
sync

echo 'SHUTDOWN WITHOUT SAVING!!!'
###end###
