#!/bin/ash
#compile nvidia kernel modules, xorg drivers, GL libraries, create sfs.
#20231107 compile inside a container.
#20231108 change individual to mainonly
#20231109 remove lib32. pass name of .run file on commandline. support 390.*
#20231116 remove:  --no-peermem --no-unified-memory
#20240826 470 remove --no-dkms

. /etc/DISTRO_SPECS
if [ -f /etc/rc.d/PUPSTATE ];then
. /etc/rc.d/PUPSTATE
else
 #running in container.
. /PUPSTATE
fi

if [ -f /NVRUN ];then #20231109
 #running in container.
 NVRUN="$(cat /NVRUN)"
 echo 'running in container...'
else
 NVRUN="$1"
fi
[ ! "$NVRUN" ] && exit 5

#setup and start the container...
if [ ! -f /PWD ];then #/PWD exist if script running inside container.
 #20231109 clean out console container...
 for aF in NVRUN PUPSTATE PWD
 do
  rm -f /mnt/wkg/containers/console/.session/${aF} 2>/dev/null
 done
 rm -rf /mnt/wkg/containers/console/.session/usr 2>/dev/null
 rm -rf /mnt/.easy_rw/console
 mkdir /mnt/.easy_rw/console
 
 #if [ ! -e /mnt/${WKG_DEV}/${WKG_DIR}containers/console/devx.sfs ];then
  sed -i '/EASY_LAYER_RO1/d' /mnt/${WKG_DEV}/${WKG_DIR}containers/console/configuration
  echo "EASY_LAYER_RO1='devx.sfs'" >> /mnt/${WKG_DEV}/${WKG_DIR}containers/console/configuration
  ln -snf ../../sfs/easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}/devx_${DISTRO_VERSION}_amd64.sfs /mnt/${WKG_DEV}/${WKG_DIR}containers/console/devx.sfs
 #fi
 mkdir -p /mnt/${WKG_DEV}/${WKG_DIR}containers/console/.session/usr/bin
 cp -a -f ./5nvidia.sh /mnt/${WKG_DEV}/${WKG_DIR}containers/console/.session/usr/bin/
 PWD="$(pwd)"
 if [ "$PWD" == "/usr/src" ];then
  PWD="$(realpath /usr/src)"
 fi
 echo -n "$PWD" > /mnt/${WKG_DEV}/${WKG_DIR}containers/console/.session/PWD
 cp -a -f /etc/rc.d/PUPSTATE /mnt/${WKG_DEV}/${WKG_DIR}containers/console/.session/PUPSTATE
 echo -n "$NVRUN" > /mnt/${WKG_DEV}/${WKG_DIR}containers/console/.session/NVRUN
 #exec /usr/local/easy_containers/ec-chroot-admin console 5nvidia.sh
 /usr/local/easy_containers/start-container console
 ./chroot-rootfs /mnt/${WKG_DEV}/${WKG_DIR}containers/console/container /usr/bin/5nvidia.sh
 /usr/local/easy_containers/stop-container console
 sync
 #erase everything created in container...
 rm -rf /mnt/.easy_rw/console
 sync
 mkdir /mnt/.easy_rw/console
 exit
fi
#############
[ ! -f /INSIDE_console ] && exit 1
PWD="$(cat /PWD)"

DEV1="$(grep '^devtmpfs /dev/s' /proc/mounts | cut -f 2 -d ' ' | tail -n 2 | sed -e 's%/dev/%%' | head -n 1)"
mkdir -p /mnt/${DEV1} #ex: sda1
busybox mount -t ext4 /dev/${DEV1} /mnt/${DEV1}
DEV2="$(grep '^devtmpfs /dev/s' /proc/mounts | cut -f 2 -d ' ' | tail -n 2 | sed -e 's%/dev/%%' | tail -n 1)"
mkdir -p /mnt/${DEV2} #
busybox mount -t ext4 /dev/${DEV2} /mnt/${DEV2}
mkdir -p /mnt/build
busybox mount -o bind /mnt/${DEV2} /mnt/build
mkdir -p /mnt/.easy_rw
busybox mount -t ext2 /dev/zram1 /mnt/.easy_rw

#/usr/bin/ash #TEST
#exit #TEST

cd $PWD
[ ! -f ./DOTconfig ] && exit 2

ln -snf ${PWD} /usr/src
cd /usr/src
[ ! -f ./DOTconfig ] && exit 3

#NVVER='525.125.06'
NVVER="$(echo -n "$NVRUN" | cut -f 4 -d '-' | sed 's%\.run%%')"
KERNELVER="`uname -r`"

mkdir -p out-k${KERNELVER}

echo
SFSPATH="/mnt/build/bk/www/ibiblio/easyos/${DISTRO_TARGETARCH}/packages/sfs/easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}"
if [ ! -d ${SFSPATH} ];then
 echo "Path ${SFSPATH} does not exist"
 exit 4
fi

#if [ ! -f NVIDIA-Linux-x86_64-${NVVER}.run ];then
# echo '/usr/src/NVIDIA-Linux-x86_64-${NVVER}.run does not exist, aborting'
# exit 5
#fi
chmod 755 ${NVRUN} #NVIDIA-Linux-x86_64-${NVVER}.run

rm -rf out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64 2>/dev/null
rm -f out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64.sfs 2>/dev/null
rm -f out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64.specs 2>/dev/null

#rem:  --no-install-libglvnd  20231109 rem:  --compat32-libdir=lib32 --compat32-prefix=/usr
case "$NVRUN" in
 NVIDIA-Linux-x86_64-390*)
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --no-unified-memory --concurrency-level=1 --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
 NVIDIA-Linux-x86_64-470*) #20240826
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --concurrency-level=1 --skip-depmod --no-systemd --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
 *)
  #20231116 remove:  --no-peermem --no-unified-memory
  CC=gcc ./${NVRUN} --force-selinux=no --run-nvidia-xconfig --expert --opengl-prefix=/usr --utility-prefix=/usr --documentation-prefix=/usr --no-nvidia-modprobe --kernel-name=${KERNELVER} --no-rpms --no-x-check --no-nouveau-check --no-dkms --concurrency-level=1 --skip-depmod --no-systemd --no-questions --ui=none --no-check-for-alternate-installs --no-kernel-module-source --no-distro-scripts --glvnd-egl-config-path=/usr/share/glvnd/egl_vendor.d --x-sysconfig-path=/usr/share/X11/xorg.conf.d
 ;;
esac

##this got changed, restore:
#cp -a -f --remove-destination /mnt/.easy_ro/easy_sfs/usr/lib/libGL.so.1 /usr/lib/

mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr
cp -a /mnt/.easy_rw/console/usr/bin out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/
cp -a /mnt/.easy_rw/console/usr/lib out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/
#cp -a /mnt/.easy_rw/console/usr/lib32 out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/
cp -a /mnt/.easy_rw/console/usr/share out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/

#20231109 old 390 .run cannot disable running depmod...
DELfiles="$(find out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER} -mindepth 1 -maxdepth 1 -type f | tr '\n' ' ')"
for aDEL in $DELfiles
do
 rm -f out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/usr/lib/modules/${KERNELVER}/${aDEL##*/}
done

##blacklist nouveau module (ref: /etc/rc.d/rc.sysinit creates blacklist.conf)...
#mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d
#cp -a /etc/rc.d/MODULESCONFIG out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d/
#sed -i -e 's%^SKIPLIST=" %SKIPLIST=" nouveau %' out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/rc.d/MODULESCONFIG

#20231109 for dual intel and nvidia pcs... throw in nouveau also, instead of SKIPLIST...
mkdir -p out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/modprobe.d
echo 'blacklist i915
install i915 /bin/false
blacklist nouveau
install nouveau /bin/false' > out-k${KERNELVER}/nvidia_${KERNELVER}-${NVVER}_amd64/etc/modprobe.d/nvidia.conf

#create sfs...
cd out-k${KERNELVER}
echo "Creating meta-data for nvidia_${KERNELVER}-${NVVER}_amd64.sfs ..."
SIZEK="$(du -s -k nvidia_${KERNELVER}-${NVVER}_amd64 | cut -f 1)"
echo "#generated by 5nvidia.sh:
DB_pkgname='nvidia_${KERNELVER}-${NVVER}_amd64'
DB_nameonly='nvidia'
DB_version='${KERNELVER}'
DB_revision='${NVVER}'
DB_arch='amd64'
DB_categories='Utility'
DB_size='${SIZEK}'
DB_fullfilename='nvidia_${KERNELVER}-${NVVER}_amd64.sfs'
DB_dependencies='easy'
DB_description='Official NVIDIA GPU support'
DB_description_long='Kernel modules and libraries for NVIDIA GPUs. 390 series is for older GPUs, 525 and later for newer.'
DB_compileddistro='${DISTRO_BINARY_COMPAT}'
DB_compiledrelease='${DISTRO_COMPAT_VERSION}'
DB_path='easyos/${DISTRO_BINARY_COMPAT}/${DISTRO_COMPAT_VERSION}'
DB_apps='nvidia'
DB_icons='nvidia'
DB_install='mainonly'
DB_hash=''" > nvidia_${KERNELVER}-${NVVER}_amd64.specs

mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/.control
cp -a nvidia_${KERNELVER}-${NVVER}_amd64.specs nvidia_${KERNELVER}-${NVVER}_amd64/.control/nvidia.specs
mkdir -p nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps
#if [ -f /usr/share/doc/NVIDIA_GLX-1.0/nvidia-settings.png ];then #20231109
# ffmpeg -i /usr/share/doc/NVIDIA_GLX-1.0/nvidia-settings.png -vf scale=48:48 nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png
#fi
#cp -a nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia.png
cp -a ../nvidia.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/
cp -a ../nvidia.png nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia48.png

sync
mksquashfs nvidia_${KERNELVER}-${NVVER}_amd64 nvidia_${KERNELVER}-${NVVER}_amd64.sfs -comp zstd
sync
chmod 644 nvidia_${KERNELVER}-${NVVER}_amd64.sfs

HASH="$(sha256sum nvidia_${KERNELVER}-${NVVER}_amd64.sfs | cut -f 1 -d ' ')"
sed -i -e "s%^DB_hash=.*%DB_hash='${HASH}'%" nvidia_${KERNELVER}-${NVVER}_amd64.specs

echo -n "Press ENTER to install SFS: "
read goon
echo
echo "nvidia_${KERNELVER}-${NVVER}_amd64.sfs created.
Installed at: ${SFSPATH}"
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.sfs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64.specs ${SFSPATH}/
cp -a -f nvidia_${KERNELVER}-${NVVER}_amd64/usr/share/pixmaps/nvidia.png ${SFSPATH}/
sync

###end###
